package wood.keith.opentools.ibxdesigner;

import java.util.Enumeration;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import com.borland.internetbeans.IxComponent;
import com.borland.internetbeans.PageProducer;
import com.borland.jbuilder.cmt.CmtComponents;
import com.borland.jbuilder.cmt.CmtComponentSource;
import com.borland.jbuilder.cmt.CmtModelNode;
import com.borland.jbuilder.cmt.CmtSubcomponent;
import com.borland.jbuilder.designer.Designer;
import com.borland.jbuilder.designer.DesignerManager;
import com.borland.jbuilder.designer.DesignerViewer;
import com.borland.primetime.PrimeTime;

/**
 * A designer that manages InternetBeans components.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  10 September 2001
 */
public class InternetBeansDesigner implements Designer, TreeSelectionListener {

  private static final String VERSION = "1.0";

  private InternetBeansModelNode _rootNode = null;
  private DesignerViewer _viewer = null;

  /**
   * Register the new designer for the InternetBeans.
   *
   * @param  majorVersion  the major version of the current OpenTools API
   * @param  minorVersion  the minor version of the current OpenTools API
   */
  public static void initOpenTool(byte majorVersion, byte minorVersion) {
    if (majorVersion != PrimeTime.CURRENT_MAJOR_VERSION) {
      return;
    }
    DesignerManager.registerDesigner(new InternetBeansDesigner());
    if (PrimeTime.isVerbose()) {
      System.out.println("Loaded InternetBeans Designer v" + VERSION);
      System.out.println("Written by Keith Wood (kbwood@iprimus.com.au)");
    }
  }

  public InternetBeansDesigner() {
  }

  /**
   * Update the viewer for this designer.
   *
   * @param  source  the source reference that corresponds to the components
   * @param  root    the root of the subtree for this designer
   * @param  node    the node being opened
   * @param  viewer  the node viewer component
   * @return  true if this node was handled, false otherwise
   */
  public boolean activate(CmtComponentSource source, CmtModelNode root,
      CmtModelNode node, DesignerViewer viewer) {
    _viewer = viewer;
    InternetBeansViewer designer =
        (InternetBeansViewer)viewer.getDesignView(this);
    if (designer == null) {
      // Create and register the UI for the designer
      designer = new InternetBeansViewer();
      viewer.setDesignView(this, designer);
    }
    // Then set up the UI to display the current node
    Object object = root.getSubcomponent().getLiveInstance();
    designer.setPageProducer(
        (object instanceof PageProducer ? (PageProducer)object : null),
        root.getSubcomponent().getName());
    object = node.getSubcomponent().getLiveInstance();
    designer.setSelection(
        object instanceof IxComponent ? (IxComponent)object : null);
    return true;
  }

  /**
   * Build a subtree for the components handled by this designer -
   * PageProducers and IxComponents from the InternetBeans collection.
   *
   * @param  source            the source reference that corresponds to the components
   * @param  componentManager  the manager of the components
   */
  public void annotate(CmtComponentSource source,
      CmtComponents componentManager) {

    // Obtain an array containing all subcomponents (bean instance variables)
    // of the class being designed.  The array will also contain "this".
    CmtSubcomponent[] subcomponents = source.getSubcomponents();

    // Create a root model node object for the subtree.
    _rootNode = new InternetBeansModelNode(this, null, null, getModelName());

    // Create a tree structure for holding the structure of our model.
    // The root object of the tree is our root node.
    DefaultTreeModel tree = new DefaultTreeModel(_rootNode);

    // Now create the "model" itself.
    InternetBeansModel model =
        new InternetBeansModel(this, source, tree, getModelName());

    // Create additional nodes, one for each PageProducer, and attach them
    // to the tree, all as first level children of the root node.
    for (int index = 0; index < subcomponents.length; index++) {
      if (subcomponents[index].getLiveInstance() instanceof PageProducer)  {
        tree.insertNodeInto(new InternetBeansModelNode(
            this, model, subcomponents[index]), _rootNode, 0);
      }
    }

    // Create additional nodes, one for each IxComponent,
    // and attach them into the tree under their page producer (if available).
    for (int index = 0; index < subcomponents.length; index++) {
      if (subcomponents[index].getLiveInstance() instanceof IxComponent)  {
        PageProducer producer =
            ((IxComponent)subcomponents[index].getLiveInstance()).
            getPageProducer();
        InternetBeansModelNode parent = _rootNode;
        Enumeration children = _rootNode.children();
        while (children.hasMoreElements()) {
          InternetBeansModelNode child =
              (InternetBeansModelNode)children.nextElement();
          if (child.getSubcomponent().getLiveInstance() == producer) {
            parent = child;
            break;
          }
        }
        tree.insertNodeInto(new InternetBeansModelNode(
            this, model, subcomponents[index]), parent, 0);
      }
    }

    // Hand the model we made back to the source. This call,
    // source.addModel(), is the primary action that annotate() performs
    // back to the ComponentTree's structure. You can add zero or more models.
    source.addModel(model, null);
  }

  /**
   * Tidy up after finished with the designer.
   *
   * @param  source  the source reference that corresponds to the components
   * @param  viewer  the node viewer component
   */
  public void close(CmtComponentSource source, DesignerViewer viewer) {
    InternetBeansViewer designer =
        (InternetBeansViewer)viewer.getDesignView(this);
    if (designer != null) {
      designer.setPageProducer(null, "");
    }
    viewer.getSelection().removeTreeSelectionListener(this);
  }

  public String getModelName() { return "InternetBeans"; }

  /**
   * Create the UI for the designer if necessary.
   *
   * @param  source  the source reference that corresponds to the components
   * @param  viewer  the node viewer component
   */
  public void open(CmtComponentSource source, DesignerViewer viewer) {
    viewer.getSelection().addTreeSelectionListener(this);
    InternetBeansViewer designer =
        (InternetBeansViewer)viewer.getDesignView(this);
    if (designer == null) {
      designer = new InternetBeansViewer();
      viewer.setDesignView(this, designer);
    }
  }

  /**
   * Update the UI to show a particular PageProducer.
   *
   * @param  producer  the PageProducer to load
   * @param  name      the component name to display
   */
  public void setPageProducer(PageProducer producer, String name) {
    InternetBeansViewer designer =
        (InternetBeansViewer)_viewer.getDesignView(this);
    if (designer != null) {
      designer.setPageProducer(producer, name);
    }
  }

  /**
   * Update the UI to select a particular IxComponent.
   *
   * @param  object  the IxComponent to select within the HTML file
   */
  public void setSelection(Object object) {
    InternetBeansViewer designer =
        (InternetBeansViewer)_viewer.getDesignView(this);
    if (designer != null) {
      designer.setSelection(
          object instanceof IxComponent ? (IxComponent)object : null);
    }
  }

  /**
   * Respond to selections within the structure tree and
   * update the UI accordingly.
   *
   * @param  evt  the event for the tree selection
   */
  public void valueChanged(TreeSelectionEvent evt) {
    if (_viewer == null) {
      return;
    }
    try {
      TreePath[] paths = evt.getPaths();
      if (paths != null && paths.length >= 1) {
        Object object = paths[0].getLastPathComponent();
        if (object instanceof InternetBeansModelNode) {
          object = ((InternetBeansModelNode)object).getSubcomponent();
          object = (object == null ? null :
            ((CmtSubcomponent)object).getLiveInstance());
          setSelection(object);
        }
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}
